package com.jiubang.commerce.chargelocker.holder;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Environment;
import android.text.TextUtils;

/***
 * 充电锁内部竞的"避让原则"
 * 
 * @author xiexingwu
 * @使用一 应用初始化new ChargeLockerHolder().applyHolder()申请充电锁权限的操作
 * @使用二 显示充电锁界面时需要调用new ChargeLockerHolder().recordShowed()方法
 * @version 9
 */
public class ChargeLockerHolder {
	private static final String SHARED_PREFERENCES_FILENAME = "charge_locker_product_holder";
	private static final String SP_KEY_HOLDER_RESULT = "holder_result";

	public String mPARENT_PATH;
	public String mPATH;

	private static int sSP_VALUE_DEFULE = 0;
	private static int sSP_VALUE_SUCCESS = 1;
	private static int sSP_VALUE_FAIL = -1;
	private Context mContext;
	private String mPackageName;
	private int mVersionCode;
	private static boolean sEFFECT = true;
	private static String sLogTag = "ChargeLockerHolder";
	private List<HolderProduct> mProducts = new ArrayList<HolderProduct>();
	// 统计的入口产品id：http://wiki.3g.net.cn/pages/viewpage.action?pageId=14254166
	private String mStatisticEntranceID = "";

	/***
	 * 
	 * @param context
	 * @param statisticEntranceID
	 *            统计的入口产品id：
	 * @param showLog
	 *            是否打印日志(由于进程关系，这个值参入即可)，true:打印，false不打印
	 * @统计协议 
	 *       http://wiki.3g.net.cn/pages/viewpage.action?pageId=14254166《产品ID对照表》
	 */
	public ChargeLockerHolder(Context context, String statisticEntranceID,
			boolean showLog) {
		this.mContext = context.getApplicationContext();
		try {
			mPARENT_PATH = Environment.getExternalStorageDirectory()
					+ File.separator + "Android" + File.separator + "data"
					+ File.separator + "com.jiubang.gochargelockor"
					+ File.separator;
			mPATH = mPARENT_PATH + "hodler.cfg";
			HolderLogUtils.printLog(showLog);
			mPackageName = mContext.getPackageName();
			mVersionCode = mContext.getPackageManager().getPackageInfo(
					mPackageName, 0).versionCode;
			mStatisticEntranceID = statisticEntranceID;
			// sLogTag = mContext.getPackageManager().getApplicationLabel(
			// mContext.getApplicationInfo())
			// + "";
			if (!sEFFECT) {
				return;
			}
			log4i("本包[包名:" + mPackageName + "版本:" + mVersionCode + "]");
			// demo
			// mProducts.add(HolderProduct.produce(HolderProductType.demoA));
			// 急加速
			mProducts.add(HolderProduct.produce(HolderProductType.ZeroZboost));
			// 应用锁
			mProducts.add(HolderProduct.produce(HolderProductType.Alock));
			// go输入法
			mProducts.add(HolderProduct.produce(HolderProductType.GOKeyboard));
			// go短信
			mProducts.add(HolderProduct.produce(HolderProductType.GOMsm));

			// 没有文件夹，就创建
			File dir = new File(mPARENT_PATH);
			if (!dir.exists()) {
				dir.mkdirs();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * 设置是否生效，默认生效，等同于功能开关使用
	 * 
	 * @注意 测试使用
	 * @param effect
	 *            是否生效，true启动生效，false关闭不生效
	 */
	@Deprecated
	public static void setEffect(boolean effect) {
		sEFFECT = effect;
	}

	/**
	 * 获取改避让逻辑是否生效(起作用)
	 * 
	 * @注意 测试使用
	 * @return true 生效，false 无效
	 */
	@Deprecated
	public static boolean getEffect() {
		return sEFFECT;
	}

	/***
	 * 测试方法，单个方法独立测试用
	 * 
	 * @注意 测试使用
	 * @return
	 */
	@Deprecated
	public boolean test() {
		boolean result = false;
		if (!sEFFECT) {
			log4i("该功能不生效");
			return result;
		}
		// 1.上次结果的保存和读取
		// saveLastResult(true);
		// getLastResult();
		// 2.旧版验证
		// checkOllVersion(new CharageLockorProduct("com.jb.helper", 4, 10));
		// 3.读取SD位置信息
		// HolderInfo holderInfo = new HolderInfo(true);
		// holderInfo.isEmpty();
		// 4.保存位置信息到SD卡
		// new HolderInfo(false).saveHolderToSDCard();

		// HolderInfo h = new HolderInfo(false);
		// h.mHolderPackageName = "com.sivun.aidl.client";
		// h.show = false;
		// h.saveHolderToSDCard();
		// // 是否安装了
		// boolean install = hasInstalled(new
		// HolderInfo(true).mHolderPackageName);
		// log_i("test=>" + "判断是否卸载" + install);
		// recordShowed();
		return result;
	}

	/**
	 * 申请"锁屏显示持有权"操作
	 * 
	 * @return true:申请成功，false：申请失败
	 */
	public boolean applyHolder() {
		if (!sEFFECT) {
			log4i("该功能不生效");
			return false;
		}
		// 1.检查是否以后有旧结果
		int lastResult = getLastResult();
		if (lastResult == sSP_VALUE_SUCCESS) {
			log4i("holder=>" + "上次结果成功");
			return true;
		} else if (lastResult == sSP_VALUE_FAIL) {
			log4i("holder=>" + "上次结果失败");
			return false;
		}
		// 2.获取位置信息
		HolderInfo holderInfo = new HolderInfo(true);
		if (holderInfo.isHolder()) {
			// 本包，表示已经强制持有
			return saveLastResult(true, false);
		}
		// 3.检查旧版
		for (HolderProduct product : mProducts) {
			if (product != null && !checkOllVersion(product)) {
				return saveLastResult(false, true);
			}
		}
		// 4.位置信息其他判断
		if (holderInfo.isEmpty()) {
			// 没有持有者信息，直接保存自己到SD卡
			new HolderInfo(false).saveHolderToSDCard();
			return saveLastResult(true, true);
		} else if (!holderInfo.isHolder()) {
			// 他包，持有者为别人，
			boolean install = hasInstalled(holderInfo.mHolderPackageName);
			if (!install) {
				// 非安装，认为被卸载了
				if (holderInfo.hasShow()) {
					// 已经有显示记录
					return saveLastResult(false, true);
				} else {
					deleteSDFile(mPATH);
					new HolderInfo(false).saveHolderToSDCard();
					return saveLastResult(true, true);
				}
			}
		}

		return saveLastResult(false, true);
	}

	/***
	 * 记录显示信息
	 */
	public void recordShowed() {
		log4i("recordShowed==>");
		if (!sEFFECT) {
			log4i("该功能不生效");
			return;
		}
		//已经抢位成功过
		boolean hasHolderSucc = getLastResult() == sSP_VALUE_SUCCESS;
		
		HolderInfo holderInfo = new HolderInfo(true);
		// 1.无持有者信息，写入自己的
		if (hasHolderSucc && holderInfo.isEmpty()) {
			HolderInfo myHolderInfo = new HolderInfo(false);
			myHolderInfo.mShow = true;
			myHolderInfo.saveHolderToSDCard();
			log4i("recordShowed==>无持有者信息，写入自己的");
			HolderBaseSeq103OperationStatistic.uploadChargeLockerHolderShow(
					mContext, mPackageName, mStatisticEntranceID);
			return;
			// 2.本包,而且没有显示过，更改显示的标记位
		} else if (hasHolderSucc && holderInfo.isHolder()
				&& !holderInfo.hasShow()) {
			holderInfo.mShow = true;
			holderInfo.saveHolderToSDCard();
			log4i("recordShowed==>本包，更改显示的标记位");
			HolderBaseSeq103OperationStatistic.uploadChargeLockerHolderShow(
					mContext, mPackageName, mStatisticEntranceID);
			return;
		}
		log4i("recordShowed==>不处理");
		// 3.本包而且显示标记位已经已经展示、他包都不做任何处理
	}

	
	/***
	 * 是否已占位信息（不做占位相关操作，只读取判断是否有或者没有）
	 * 
	 * @return true有，false没有
	 */
	public boolean haveHolder() {
		HolderInfo holderInfo = new HolderInfo(true);
		return !holderInfo.isEmpty();
	}
	
	/***
	 * 是否已占位信息中的包名（不做占位相关操作，只读取判断是否有或者没有）
	 * 
	 * @return 包名，或者null
	 */
	public String getHolderPackageName() {
		HolderInfo holderInfo = new HolderInfo(true);
		if(holderInfo.isEmpty()){
			return null;
		}else{
			return holderInfo.mHolderPackageName;
		}
	}
	
	/***
	 * 检查旧版本后的结果
	 * 
	 * @return true:无旧版冲突问题，false:与旧版有冲突问题
	 */
	private boolean checkOllVersion(HolderProduct charageLockorProduct) {
		log4i("checkOllVersion==>");
		boolean result = false;
		if (!sEFFECT) {
			log4i("该功能不生效");
			return result;
		}
		PackageManager pm = mContext.getPackageManager();
		try {
			// 获取指定的已安装的app信息，注意：如果获取不到会抛出NameNotFoundException异常，需要扑捉处理
			PackageInfo info = pm.getPackageInfo(
					charageLockorProduct.mPackageName,
					PackageManager.GET_CONFIGURATIONS);
			if (info != null) {
				log4i("checkOllVersion==>已经安装:"
						+ charageLockorProduct.mPackageName);
				result = charageLockorProduct.check(info.packageName,
						info.versionCode);
			}
		} catch (PackageManager.NameNotFoundException e) {
			log4i("checkOllVersion==>没安装:" + charageLockorProduct.mPackageName);
			result = true;
		}
		log4i("checkOllVersion==>旧版冲突检测:" + (result ? "无冲突" : "冲突")
				+ charageLockorProduct.mPackageName);
		return result;
	}

	/***
	 * 根据包名判断，是否已经安装到手机
	 * 
	 * @param packageName
	 * @return true已经安装，false未安装
	 */
	private boolean hasInstalled(String packageName) {
		log4i("hasInstalled==>");
		PackageManager pm = mContext.getPackageManager();
		try {
			PackageInfo info = pm.getPackageInfo(packageName,
					PackageManager.GET_CONFIGURATIONS);
			if (info != null) {
				log4i("hasInstalled==>已经安装了" + packageName);
				return true;
			} else {
				log4i("hasInstalled==>没安装" + packageName);
				return false;
			}
		} catch (PackageManager.NameNotFoundException e) {
			log4i("hasInstalled==>没安装" + packageName);
			return false;
		}
	}

	/***
	 * 接入充电锁的apk，需要规避的应用,基类
	 */
	// private class CharageLockorProduct {}

	/**
	 * 锁屏显示持有者信息
	 */
	private class HolderInfo {
		// 锁屏权利持有者的文件信息
		private File mFile;
		// 锁屏权利持有者的包名
		private String mHolderPackageName = "";
		// 锁屏权利持有者是否已经展示过
		private boolean mShow;
		private String mSPLIT_KEY = ":";
		private String mTRUE = "true";
		private String mFALSE = "false";
		// 配置
		private String mCfg;

		HolderInfo(boolean fromSDCard) {
			if (fromSDCard) {
				parse();
				log4i("HolderInfo==>读取配置[持有者:"
						+ (isEmpty() ? "无" : mHolderPackageName) + ",展示:"
						+ hasShow() + "]");
			} else {
				mFile = new File(mPATH);
				mHolderPackageName = mPackageName;
				log4i("HolderInfo==>创建配置[持有者:"
						+ (isEmpty() ? "无" : mHolderPackageName) + ",展示:"
						+ hasShow() + "]");
			}
		}

		/***
		 * 是否有充电锁显示权的持有者信息
		 */
		private boolean isEmpty() {
			return mFile == null;
		}

		/**
		 * 是否 有持有者的权利
		 * 
		 * @return
		 */
		private boolean isHolder() {
			log4i("isHolder==>[持有者:" + mHolderPackageName + ",本包:"
					+ mPackageName + "]");
			return mHolderPackageName.equals(mPackageName);
		}

		/***
		 * 持有者，是本包名吗？被改过，格式不对都认为是其他包
		 * 
		 * @return
		 */
		private boolean hasShow() {
			return mShow;
		}

		/**
		 * 保存电充电锁显示权利持有者信息到SD卡
		 */
		private void saveHolderToSDCard() {
			// 可多次调用，每次都是覆盖操作
			log4i("saveHolderToSDCard==>:"
					+ (mShow ? mHolderPackageName + mSPLIT_KEY + mTRUE
							: mHolderPackageName + mSPLIT_KEY + mFALSE));
			outHolder(mShow ? mHolderPackageName + mSPLIT_KEY + mTRUE
					: mHolderPackageName + mSPLIT_KEY + mFALSE);
		}

		/**
		 * 对持有文件进行解析处理
		 */
		private void parse() {
			// 标准格式：【包名:显示记录】，
			// 其中包名：的匹配需要完全一致;
			// 显示记录(忽略大小写)：true代表已经显示过，false或者其他都代表还没展示过
			// 如:com.jiubang.goms:false,错误文件，可当成无效处理
			mCfg = getSdCardFileString(mPATH);
			if (!TextUtils.isEmpty(mCfg) && mCfg.contains(mSPLIT_KEY)) {
				String[] sts = mCfg.split(mSPLIT_KEY);
				if (sts.length >= 2) {
					// 之所以允许大于2，是为了后续可以扩展
					mHolderPackageName = sts[0];
					mShow = sts[1].equalsIgnoreCase(mTRUE);
					mFile = new File(mPATH);
					return;
				}
				// 说明记录已经被篡改，可当无效处理
				mCfg = null;
			}
		}
	}

	/***
	 * 保存上次的结果
	 * 
	 * @param success
	 *            申请是否成功
	 * @param needStatistic
	 *            是否需要统计本次行为
	 * @return 直接返回传入的参数
	 */
	private boolean saveLastResult(boolean success, boolean needStatistic) {
		log4i("saveLastResult==>");
		SharedPreferences sp = getSharedPreferences();
		if (sp != null) {
			log4i("saveLastResult==>保存处理的结果:"
					+ (success ? sSP_VALUE_SUCCESS : sSP_VALUE_FAIL));
			sp.edit()
					.putInt(SP_KEY_HOLDER_RESULT,
							success ? sSP_VALUE_SUCCESS : sSP_VALUE_FAIL)
					.commit();
		}

		// 统计
		if (needStatistic) {
			HolderBaseSeq103OperationStatistic.uploadChargeLockerHolder(
					mContext, mPackageName, mStatisticEntranceID, success);
		}
		return success;
	}

	/**
	 * 获取上次结果
	 * 
	 * @return
	 * @see #sSP_VALUE_DEFULE 默认，无数据
	 * @see #sSP_VALUE_SUCCESS 成功
	 * @see #sSP_VALUE_FAIL 失败
	 */
	private int getLastResult() {
		SharedPreferences sp = getSharedPreferences();
		if (sp == null) {
			return sSP_VALUE_DEFULE;
		}
		int result = sp.getInt(SP_KEY_HOLDER_RESULT, sSP_VALUE_DEFULE);
		log4i("getLastResult=>" + "上次结果值:" + result);
		return result;
	}

	private SharedPreferences getSharedPreferences() {
		SharedPreferences sp;
		if (Build.VERSION.SDK_INT >= 11) {
			sp = mContext.getSharedPreferences(SHARED_PREFERENCES_FILENAME,
					Context.MODE_MULTI_PROCESS);
		} else {
			sp = mContext.getSharedPreferences(SHARED_PREFERENCES_FILENAME,
					Context.MODE_PRIVATE);
		}
		return sp;
	}

	/***
	 * 文件中读取为String
	 */
	public String getSdCardFileString(String filePath) {
		InputStream ins = null;

		try {
			ins = new FileInputStream(filePath);
		} catch (Exception e) {
			return "";
		}
		return transInputSteam2String(ins);
	}

	/**
	 * 流转成字符串
	 */
	private String transInputSteam2String(InputStream ins) {
		String dataStr = null;
		try {
			int a = ins.available();
			if (a == 0) {
				return null;
			}

			byte[] data = new byte[a];
			ins.read(data);
			dataStr = new String(data, "UTF-8");
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		} finally {
			try {
				ins.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		return dataStr;
	}

	/***
	 * 文本到文件出
	 * 
	 * @param outStr
	 * @return
	 */
	private boolean outHolder(String outStr) {
		log4i("outHolder==>");
		File file = new File(mPATH);
		if (file.exists()) {
			file.delete();
		}
		log4i("outHolder==>1");
		BufferedWriter bw = null;
		try {
			log4i("outHolder==>2");
			bw = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(file.getAbsoluteFile()), "UTF-8"));
			log4i("outHolder==>3");
			bw.write(outStr);
			log4i("outHolder==>" + outStr);
		} catch (IOException e) {
			log4i("outHolder==>报错：" + e.getLocalizedMessage());
			e.printStackTrace();
		} finally {
			if (bw != null) {
				try {
					bw.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return true;
	}

	/**
	 * 根据路径删除SD卡文件
	 * 
	 * @param filePath
	 */
	public File deleteSDFile(String filePath) {
		File file = new File(filePath);
		if (file.exists()) {
			file.delete();
		}
		return file;
	}

	/**
	 * 打印日志
	 * 
	 * @param l
	 */
	public static void log4i(String l) {
		HolderLogUtils.i(sLogTag, l);
	}
}
