package com.jiubang.commerce.chargelocker.holder;

import java.util.HashMap;
import java.util.Map;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.text.TextUtils;

import com.gau.go.gostaticsdk.StatisticsManager;
import com.gau.go.gostaticsdk.beans.OptionBean;

/**
 * GoLauncher统计基类
 * 
 * @author caoyaming
 *
 */
public abstract class HolderAbsBaseStatistic {
	// 操作统计-日志序列
	public static final int OPERATION_LOG_SEQ_105 = 105;
	// ======================产品ID======================
	public static int sPRODUCT_ID = -1;
	// LOG_TAG
	protected static final String LOG_TAG = "CommerceSDKStatistic";
	// 操作结果---操作成功
	public final static int OPERATE_SUCCESS = 1;
	// 操作结果---操作失败
	public final static int OPERATE_FAIL = 0;
	// 拼装上传参数分隔符
	protected static final String STATISTICS_DATA_SEPARATE_STRING = "||";
	// 布局统计数据分割符
	protected static final String LAYOUT_STATISTICS_DATA_SEPARATE = "#";
	// 保存统计数据的分割符
	protected static final String STATISTICS_DATA_SEPARATE_ITEM = "#";
	// PrivatePreference
	private static SharedPreferences sPrivatePreference;
	// 下载与安装成功的最大间隔时间(半小时)
	protected static final long DOWNLOAD_TO_INSTALL_MAX_INTERVAL_TIME = 30 * 60 * 1000;
	// 保存预安装应用信息Key前缀
	private static final String INSTALL_PREFIX_KEY = "INSTALL#";
	// 保存激活应用信息Key前缀
    private static final String ACTIVATE_PREFIX_KEY = "ACTIVATE#";
	// 缓存预安装/卸载/激活包信息key值
    public static final int FUN_ID = 0; //统计功能ID
    public static final int SENDER = FUN_ID + 1; //统计对象
    public static final int PACKAGENAME = SENDER + 1; // 应用包名
    public static final int OPTION_CODE = PACKAGENAME + 1; // 安装统计操作代码
    public static final int ENTRANCE = OPTION_CODE + 1; // 入口值
    public static final int TAB_CATEGORY = ENTRANCE + 1; // Tab分类
    public static final int POSITION = TAB_CATEGORY + 1; // 位置
    public static final int ASSOCIATED_OBJ = POSITION + 1; // 关联对象
    public static final int ADVERT_ID = ASSOCIATED_OBJ + 1; // 广告ID
    public static final int REMARK = ADVERT_ID + 1; // 备注信息
    public static final int DOWNLOAD_TIME = REMARK + 1; // 开始下载/卸载时间
    public static final int CALL_URL = DOWNLOAD_TIME + 1; //回调地址
	// 存放应用安装信息Map
	protected static Map<Integer, String> sMPreInstallMap = null;
	// 存放应用激活信息Map
    protected static Map<Integer, String> sMPreActiveMap = null;
	/**
	 * 调用统计SDK上传统计数据
	 * 
	 * @param context
	 * @param logSequence
	 *            日志序列号
	 * @param data
	 *            不保护基础数据(在SDK中做了处理)的统计数据
	 */
	protected static void uploadStatisticData(final Context context, int logSequence, int funId, final StringBuffer data, Object ...immediately) {
		//判断是否打印Log
		if (HolderLogUtils.sIS_SHOW_LOG) {
			//打印Log
			StatisticsManager.getInstance(context).enableLog(true);
		}
		// UtilTool.log(LOG_TAG, "uploadStatisticData(" + StatisticsManager.getInstance(context).getCtrlInfo(funId) + ", " + StringUtil.toString(data) + ")");
		HolderLogUtils.d(LOG_TAG, "uploadStatisticData(" + toString(data) + ")");
		// 测试
		//StatisticsManager.getInstance(context).setDebugMode();
		// 上传统计数据
		if (isAnywayImmediatelyUpload(funId)) {
			//不受开关影响,立即上传统计数据.
			StatisticsManager.getInstance(context).uploadStaticDataForOptions(logSequence, funId, toString(data), null, 
				new OptionBean(OptionBean.OPTION_INDEX_IMMEDIATELY_ANYWAY, true));
		} else {
			if (immediately.length > 0) {
				StatisticsManager.getInstance(context).uploadStaticDataForOptions(logSequence, funId, toString(data), null,
					new OptionBean(OptionBean.OPTION_INDEX_IMMEDIATELY_CARE_SWITCH, (Boolean) immediately[0]));
			} else {
				StatisticsManager.getInstance(context).uploadStaticData(logSequence, funId, toString(data), null);
			}
		}
	}
	//===================================================安装相关统计(Begin)================================================
	/**
	 * 保存预安装信息
	 * 
	 * @param context
	 * @param funId 统计功能ID
	 * @param sender 统计对象Id
	 * @param packageName 应用包名
	 * @param optionCode 统计操作码
	 * @param entrance 入口
	 * @param tabCategory Tab分类
	 * @param position 位置
	 * @param associatedObj 关联对象
	 * @param aId 广告ID
	 * @param remark 备注信息
	 * @param seqId 协议号
	 * @param callUrl 回调地址
	 */
	public static void saveReadyInstallList(Context context, int funId, String sender, String packageName, String optionCode, String entrance, String tabCategory, 
		String position, String associatedObj, String aId, String remark, String seqId, String callUrl) {
		if (TextUtils.isEmpty(optionCode) || TextUtils.isEmpty(packageName)) {
			return;
		}
		StringBuffer sb = new StringBuffer();
		//统计功能ID
		sb.append(funId).append(STATISTICS_DATA_SEPARATE_ITEM);
		//统计对象
		sb.append(sender).append(STATISTICS_DATA_SEPARATE_ITEM);
		//应用包名
		sb.append(packageName).append(STATISTICS_DATA_SEPARATE_ITEM);
		//安装统计操作代码
		sb.append(optionCode).append(STATISTICS_DATA_SEPARATE_ITEM);
		//入口
		sb.append(entrance).append(STATISTICS_DATA_SEPARATE_ITEM);
		//Tab分类
		sb.append(tabCategory).append(STATISTICS_DATA_SEPARATE_ITEM);
		//位置
		sb.append(position).append(STATISTICS_DATA_SEPARATE_ITEM);
		//关联对象
		sb.append(associatedObj).append(STATISTICS_DATA_SEPARATE_ITEM);
		//广告ID
		sb.append(aId).append(STATISTICS_DATA_SEPARATE_ITEM);
		//备注信息
		sb.append(remark).append(STATISTICS_DATA_SEPARATE_ITEM);
		//进入下载页的时间
		sb.append(System.currentTimeMillis()).append(STATISTICS_DATA_SEPARATE_ITEM);
		//回调地址
		sb.append(callUrl);
		//保存数据
		getPrivatePreference(context);
		if (sPrivatePreference != null) {
			Editor editor = sPrivatePreference.edit();
			editor.putString(INSTALL_PREFIX_KEY + seqId + packageName, sb.toString());
			editor.commit();
		}
	}
	/**
	 * 获取安装信息
	 * 
	 * @param context
	 * @param packageName 应用包名
	 * @param seqId 协议号
	 * @return
	 */
	public static String[] getInstallData(Context context, String packageName, String seqId) {
		String[] datas = null;
		// 保存数据
		getPrivatePreference(context);
		if (sPrivatePreference != null) {
			String strData = sPrivatePreference.getString(INSTALL_PREFIX_KEY + seqId + packageName, "");
			if (!TextUtils.isEmpty(strData)) {
				datas = strData.split(STATISTICS_DATA_SEPARATE_ITEM);
				// 获取成功后删除该应用预安装信息.
				Editor editor = sPrivatePreference.edit();
				editor.putString(INSTALL_PREFIX_KEY + seqId + packageName, "");
				editor.commit();
			}
		}
		return datas;
	}

	/**
	 * 是否是预安装状态
	 * 
	 * @param context
	 * @param pkgName 应用包名
	 * @param seqId 协议号
	 * @return
	 */
	public synchronized static boolean isPreInstallState(Context context, String pkgName, String seqId) {
		boolean flag = false;
		String[] data = getInstallData(context, pkgName, seqId);
		if (null != data && data.length > 1) {
			flag = true;
			// 清除缓存数据
			if (sMPreInstallMap == null) {
				sMPreInstallMap = new HashMap<Integer, String>();
			}
			sMPreInstallMap.clear();
			//统计功能ID
			sMPreInstallMap.put(FUN_ID, toString(data[0]));
			//统计对象
			sMPreInstallMap.put(SENDER, data[1]);
			//应用包名
			sMPreInstallMap.put(PACKAGENAME, data.length > 2 ? data[2] : "");
			//安装统计操作代码
			sMPreInstallMap.put(OPTION_CODE, data.length > 3 ? data[3] : "");
			//入口值
			sMPreInstallMap.put(ENTRANCE, data.length > 4 ? data[4] : "");
			//Tab分类
			sMPreInstallMap.put(TAB_CATEGORY, data.length > 5 ? data[5] : "");
			//位置
			sMPreInstallMap.put(POSITION, data.length > 6 ? data[6] : "");
			//关联对象
			sMPreInstallMap.put(ASSOCIATED_OBJ, data.length > 7 ? data[7] : "");
			//广告ID
			sMPreInstallMap.put(ADVERT_ID, data.length > 8 ? data[8] : "");
			//备注信息
			sMPreInstallMap.put(REMARK, data.length > 9 ? data[9] : "");
			//开始下载时间
			sMPreInstallMap.put(DOWNLOAD_TIME, data.length > 10 ? data[10] : "0");
			//回调地址
			sMPreInstallMap.put(CALL_URL, data.length > 11 ? data[11] : "");
		}
		return flag;
	}
	//===================================================安装相关统计(End)================================================
	//===================================================激活相关统计(start)==============================================
    /**
     * 保存预激活信息
     * 
     * @param context
     * @param funId 统计功能ID
     * @param sender 统计对象Id
     * @param packageName 应用包名
     * @param optionCode 统计操作码
     * @param entrance 入口
     * @param tabCategory Tab分类
     * @param position 位置
     * @param associatedObj 关联对象
     * @param aId 广告ID
     * @param remark 备注信息
     * @param seqId 协议号
     * @param callUrl 回调地址
     */
    public static void saveReadyActivateList(Context context, int funId, String sender, String packageName, String optionCode, String entrance, String tabCategory, 
        String position, String associatedObj, String aId, String remark, String seqId, String callUrl) {
        if (TextUtils.isEmpty(optionCode) || TextUtils.isEmpty(packageName)) {
            return;
        }
        StringBuffer sb = new StringBuffer();
        //统计功能ID
        sb.append(funId).append(STATISTICS_DATA_SEPARATE_ITEM);
        //统计对象
        sb.append(sender).append(STATISTICS_DATA_SEPARATE_ITEM);
        //应用包名
        sb.append(packageName).append(STATISTICS_DATA_SEPARATE_ITEM);
        //安装统计操作代码
        sb.append(optionCode).append(STATISTICS_DATA_SEPARATE_ITEM);
        //入口
        sb.append(entrance).append(STATISTICS_DATA_SEPARATE_ITEM);
        //Tab分类
        sb.append(tabCategory).append(STATISTICS_DATA_SEPARATE_ITEM);
        //位置
        sb.append(position).append(STATISTICS_DATA_SEPARATE_ITEM);
        //关联对象
        sb.append(associatedObj).append(STATISTICS_DATA_SEPARATE_ITEM);
        //广告ID
        sb.append(aId).append(STATISTICS_DATA_SEPARATE_ITEM);
        //备注信息
        sb.append(remark).append(STATISTICS_DATA_SEPARATE_ITEM);
        //进入下载页的时间
        sb.append(System.currentTimeMillis()).append(STATISTICS_DATA_SEPARATE_ITEM);
        //回调地址
        sb.append(callUrl);
        //保存数据
        getPrivatePreference(context);
        if (sPrivatePreference != null) {
            Editor editor = sPrivatePreference.edit();
            editor.putString(ACTIVATE_PREFIX_KEY + seqId + packageName, sb.toString());
            editor.commit();
        }
    }
    
    /**
     * 获取激活信息
     * 
     * @param context
     * @param packageName 应用包名
     * @param seqId 协议号
     * @return
     */
    public static String[] getActivateData(Context context, String packageName, String seqId) {
        String[] datas = null;
        // 保存数据
        getPrivatePreference(context);
        if (sPrivatePreference != null) {
            String strData = sPrivatePreference.getString(ACTIVATE_PREFIX_KEY + seqId + packageName, "");
            if (!TextUtils.isEmpty(strData)) {
                datas = strData.split(STATISTICS_DATA_SEPARATE_ITEM);
                // 获取成功后删除该应用预安装信息.
                Editor editor = sPrivatePreference.edit();
                editor.putString(ACTIVATE_PREFIX_KEY + seqId + packageName, "");
                editor.commit();
            }

        }
        return datas;
    }

    /**
     * 是否是预激活状态
     * 
     * @param context
     * @param pkgName 应用包名
     * @param seqId 协议号
     * @return
     */
    public synchronized static boolean isPreActivateState(Context context, String pkgName, String seqId) {
        boolean flag = false;
        String[] data = getActivateData(context, pkgName, seqId);
        if (null != data && data.length > 1) {
            flag = true;
            // 清除缓存数据
            if (sMPreActiveMap == null) {
                sMPreActiveMap = new HashMap<Integer, String>();
            }
            sMPreActiveMap.clear();
            //设置预激活数据到Map中
            setPreActivateDataToMap(sMPreActiveMap, data);
        }
        return flag;
    }
    
    /**
     * 将预激活数据设置到Map中
     * @param preActiveMap 存放数据的Map
     * @param activateDataArray 激活应用数据
     */
    private static void setPreActivateDataToMap(Map<Integer, String> preActiveMap, String[] activateDataArray) {
    	if (preActiveMap == null) {
    		preActiveMap = new HashMap<Integer, String>();
    	}
    	 //统计功能ID
    	preActiveMap.put(FUN_ID, toString(activateDataArray[0]));
        //统计对象
    	preActiveMap.put(SENDER, activateDataArray[1]);
        //应用包名
    	preActiveMap.put(PACKAGENAME, activateDataArray.length > 2 ? activateDataArray[2] : "");
        //安装统计操作代码
    	preActiveMap.put(OPTION_CODE, activateDataArray.length > 3 ? activateDataArray[3] : "");
        //入口值
    	preActiveMap.put(ENTRANCE, activateDataArray.length > 4 ? activateDataArray[4] : "");
        //Tab分类
    	preActiveMap.put(TAB_CATEGORY, activateDataArray.length > 5 ? activateDataArray[5] : "");
        //位置
    	preActiveMap.put(POSITION, activateDataArray.length > 6 ? activateDataArray[6] : "");
        //关联对象
    	preActiveMap.put(ASSOCIATED_OBJ, activateDataArray.length > 7 ? activateDataArray[7] : "");
        //广告ID
    	preActiveMap.put(ADVERT_ID, activateDataArray.length > 8 ? activateDataArray[8] : "");
        //备注信息
    	preActiveMap.put(REMARK, activateDataArray.length > 9 ? activateDataArray[9] : "");
        //开始下载时间
    	preActiveMap.put(DOWNLOAD_TIME, activateDataArray.length > 10 ? activateDataArray[10] : "0");
        //回调地址
    	preActiveMap.put(CALL_URL, activateDataArray.length > 11 ? activateDataArray[11] : "");
    }
    
    /**
     * 获取预激活数据(不从PrivatePreference中删除)
     * @param context
     * @param pkgName 包名
     * @return
     */
    public static Map<Integer, String> getPreActivateData(Context context, String pkgName) {
    	  String[] datas = null;
          // 保存数据
          getPrivatePreference(context);
          if (sPrivatePreference != null) {
              String strData = sPrivatePreference.getString(ACTIVATE_PREFIX_KEY + OPERATION_LOG_SEQ_105 + pkgName, "");
              if (!TextUtils.isEmpty(strData)) {
                  datas = strData.split(STATISTICS_DATA_SEPARATE_ITEM);
              }
          }
          //保存预激活应用数据
          Map<Integer, String> preActiveDataMap = null;
          if (datas != null && datas.length > 0) {
        	  preActiveDataMap = new HashMap<Integer, String>();
        	  //设置预激活数据到Map中
              setPreActivateDataToMap(preActiveDataMap, datas);
          }
          return preActiveDataMap;
    } 
    //===================================================激活相关统计(End)==============================================
	/**
	 * 调用统计sdk上传统计数据（目前主要只用于添加的特定地理位置上传）
	 * 
	 * @param context
	 * @param logSequence
	 * @param funId
	 * @param data
	 * @param position
	 */
	protected static void uploadStatisticDataAndLocation(final Context context, int logSequence, int funId, final StringBuffer data, final String position) {
		StatisticsManager.getInstance(context).uploadStaticDataForOptions(logSequence, funId, toString(data), null, new OptionBean(OptionBean.OPTION_INDEX_POSITION, position));
	}

	/**
	 * 是否屏蔽开关的阻碍,直接并且立即上传统计数据.
	 * @param funId 功能ID
	 * @return
	 */
	private static boolean isAnywayImmediatelyUpload(int funId) {
		if (funId == sPRODUCT_ID) {
			//只要是当前产品ID,则不受开关控制.
			return true;
		}
		return false;
	}
	
	/**
	 * 获取PrivatePreference
	 * @return
	 */
	public synchronized static SharedPreferences getPrivatePreference(Context context) {
		if (sPrivatePreference == null) {
			// 获取PrivatePreference
			sPrivatePreference = context.getSharedPreferences("ad_sdk_statistic", Context.MODE_PRIVATE);
		}
		return sPrivatePreference;
	}
	
	/**
	 * 将Object类型转换为String,且去除空格
	 * @param obj
	 * @return
	 */
	public static String toString(Object obj) {
		if (obj == null) {
			obj = "";
		}
		return obj.toString().trim();
	}
}
