package com.jiubang.commerce.chargelocker.holder;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import android.os.Handler;
import android.os.Looper;

/**
 *	目前只用于主线程的调用
 */
public class HolderAdSdkThreadExecutorProxy {
	/**
	 * 主线程Handler
	 */
	private static Handler sMainHandler;
	private static ExecutorService sExecutorService = null;

	/**
	 * 初始化
	 */
	public static void init() {
		if (sMainHandler != null) {
			return;
		}
		sMainHandler = new Handler(Looper.getMainLooper());
	}
	
	/**
	 * @param size 线程池大小
	 */
	public static void setThreadPoolSize(final int size) {
		if (size < 1) {
			return;
		}
		if (sExecutorService != null) {
			sExecutorService.shutdown();
		}
		sExecutorService = Executors.newFixedThreadPool(size);
	}
	
	/**
	 * 异步线程池中运行
	 * @param runnable
	 * @return 是否成功提交任务
	 */
	public static boolean execute(Runnable runnable) {
		if (null == sExecutorService || null == runnable) {
			return false;
		}
		sExecutorService.execute(runnable);
		return true;
	}

	/**
	 * 取消指定的任务
	 * @param task
	 */
	public static void cancel(final Runnable task) {
		if (sMainHandler == null) {
			return;
		}
		sMainHandler.removeCallbacks(task);
	}

	/**
	 * 销毁对象
	 */
	public static void destroy() {
		if (sMainHandler == null) {
			return;
		}
		sMainHandler.removeCallbacksAndMessages(null);
	}

	/**
	 * 提交一个Runable到主线程队列
	 */
	public static void runOnMainThread(Runnable r) {
		if (sMainHandler == null) {
			init();
		}
		sMainHandler.post(r);
	}

	/**
	 * 提交一个Runable到主线程队列
	 */
	public static void runOnMainThread(Runnable r, long delay) {
		if (sMainHandler == null) {
			init();
		}
		sMainHandler.postDelayed(r, delay);
	}
}
